(function() {
    // Backup original methods
    const originalElementQuerySelector = Element.prototype.querySelector;
    const originalElementQuerySelectorAll = Element.prototype.querySelectorAll;
    const originalDocumentQuerySelector = Document.prototype.querySelector;
    const originalDocumentQuerySelectorAll = Document.prototype.querySelectorAll;

    function emulateIs(selector, context) {
        const isRegex = /:is\(([^)]+)\)/;

        if (!isRegex.test(selector)) {
            return selector;
        }

        let transformedSelector = selector;
        const matchedElements = new Set();

        let match;
        while ((match = isRegex.exec(transformedSelector)) !== null) {
            const options = match[1].split(',').map(s => s.trim());

            const [beforeIs, afterIs] = transformedSelector.split(match[0]);

            const baseSelector = beforeIs.trim() || '*';
            const nodeList = context.querySelectorAll(baseSelector);

            nodeList.forEach(element => {
                if (options.some(option => {
                    return element.matches(option);
                })) {
                    matchedElements.add(element);
                }
            });

            transformedSelector = afterIs.trim();
        }

        return Array.from(matchedElements);
    }

    Document.prototype.querySelector = Element.prototype.querySelector = function(selector) {
        const context = this;
        const transformed = emulateIs(selector, context);
        console.log('querySelector', selector, transformed)
        if (typeof transformed === 'string') {
            return (this instanceof Document)
                ? originalDocumentQuerySelector.call(this, transformed)
                : originalElementQuerySelector.call(this, transformed);
        }
        return transformed[0] || null;
    };

    Document.prototype.querySelectorAll = Element.prototype.querySelectorAll = function(selector) {
        const context = this;
        const transformed = emulateIs(selector, context);
        console.log('querySelectorAll', selector, transformed)
        if (typeof transformed === 'string') {
            return (this instanceof Document)
                ? originalDocumentQuerySelectorAll.call(this, transformed)
                : originalElementQuerySelectorAll.call(this, transformed);
        }
        return transformed;
    };

    console.log('querySelector and querySelectorAll have been overridden');
})();
